[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
![Coverage: 100%](https://img.shields.io/badge/coverage-100%25-brightgreen)

# UT JWT

A package for encoding, decoding, and validating JWTs for Univers Tech applications.

Currently only supports app-to-app JWTs, as our typical authentication happens 
through AWS Cognito.

## Installation

In your requirements.txt, add a line that looks like this:

`git+ssh://git@github.com/Univers-Technology/ut_jwt@ddb2068#egg=ut_jwt`

Where the part after `ut_jwt@` can refer to a commit hash or branch. (Once this repo is
more stable, it would be nice to version it appropriately.)

## Usage

### Decoding JWTs

The `UTToken` constructor automatically validates keys according to the criteria you 
initialize the instance with. Here's the expected way to initialize `UTToken`:

    UTToken(
        token=raw_token,
        accepted_issuers=[list_of_valid_app_names],
        expected_audience=name_of_this_app,
        path_to_public_keys=https://public_uri_where_public_keys_are_stored
    )

### Encoding JWTs

Initialize an instance of `UTToken`, and give it the private key you want to use for
signing:

    token = UTToken(private_key=PRIVATE_KEY)

Then pass in the values you want to use for the `iss` and `aud` claims to the 
`create_payload()` method:

    token.create_payload("name_of_this_app", "name_of_recipient_app")

This method automatically calculates and attaches `iat` and `exp` claims, and also
assigns a value for the `kid` header to be the same as the value for the `iss` claim.
(In other words, the issuer is also going to be the expected name for the public key
that the receiver will look up later. This assumption may change in the future if we
need it to be named differently, but for now it seems convenient.)

### Tests

You can find more example usage and expected behavior within the `tests/` module. You
can run tests like this (from your terminal):

`python -m unittest`

Don't forget to install test dependencies! 

`pip install -e .\[test\]  # (Apparently macOS wants you to escape square brackets)` 

You also may find it useful to generate a coverage report:

`python -m coverage run -m unittest`

You can then retrieve it with:

`python -m coverage report`

or, for an interactive HTML version:

`python -m coverage html`

## Generating a new public/private key pair

When you have a new application/service, you'll need to create a new private key (for
signing outgoing JWTs) and public key (for other apps to verify your incoming JWTs).

### macOS

_Generate private key (don't enter a passphrase)_

`ssh-keygen -t rsa -b 4096 -m PEM -f jwt_test.key`

_Generate public key_

`openssl rsa -in jwt_test.key -pubout -outform PEM -out jwt_test.key.pub`
